/* Complex.h

    Complex Number Library - Include File
    class Complex:  declarations for Complex numbers.

All function names, member names, and operators have been borrowed
from AT&T C++, except for the addition of:

    friend Complex  acos(Complex  &);
    friend Complex  asin(Complex  &);
    friend Complex  atan(Complex  &);
    friend Complex  log10(Complex  &);
    friend Complex  tan(Complex  &);
    friend Complex  tanh(Complex  &);
    Complex  operator+();
    Complex  operator-();
*/
#pragma once
#include <math.h>
#include <string>
#include <iostream>
using namespace std;

class Complex {

public:
    // constructors
     Complex(double re_val, double im_val=0);
     Complex();

     // member access
     double &real() { return re; }
     double &imag() { return im; }

     static const double dgrad;

     // object manipulation
     double mag() { return sqrt(re*re+im*im); }
     double angle() { return dgrad*atan2(im,re); }

    // Complex manipulations
    friend double  real(Complex &);  // the real part
    friend double  imag(Complex &);  // the imaginary part
    friend Complex conj(Complex &);  // the Complex conjugate
    friend double  norm(Complex &);  // the square of the magnitude
    friend double  arg(Complex &);   // the angle in the plane

    // Create a Complex object given polar coordinates
    static Complex   polar(double mag, double angle=0);

    // Overloaded ANSI C math functions
    friend double    abs(Complex &);
    friend Complex   acos(Complex &);
    friend Complex   asin(Complex &);
    friend Complex   atan(Complex &);
    friend Complex   cos(Complex &);
    friend Complex   cosh(Complex &);
    friend Complex   exp(Complex &);
    friend Complex   log(Complex &);
    friend Complex   log10(Complex &);
    friend Complex   pow(Complex &base, double expon);
    friend Complex   pow(double base, Complex  &expon);
    friend Complex   pow(Complex  &base, Complex  &expon);
    friend Complex   sin(Complex &);
    friend Complex   sinh(Complex &);
    friend Complex   sqrt(Complex &);
    friend Complex   tan(Complex &);
    friend Complex   tanh(Complex &);

    // Binary Operator Functions
    friend Complex   operator+(Complex const &, Complex const &);
    friend Complex   operator+(double , Complex const &);
    friend Complex   operator+(Complex const &, double);
    friend Complex   operator-(Complex const &, Complex const &);
    friend Complex   operator-(double, Complex const &);
    friend Complex   operator-(Complex const &, double);
    friend Complex   operator*(Complex const &, Complex const &);
    friend Complex   operator*(double , Complex const &);
    friend Complex   operator*(Complex const &, double);
    friend Complex   operator/(Complex const &, Complex const &);
    friend Complex   operator/(Complex const &, double);
    friend Complex   operator/(double, Complex const &);
    friend int   operator==(Complex const &, Complex const &);
    friend int   operator!=(Complex const &, Complex const &);
    Complex &  operator+=(Complex &);
    Complex &  operator+=(double);
    Complex &  operator-=(Complex &);
    Complex &  operator-=(double);
    Complex &  operator*=(Complex &);
    Complex &  operator*=(double);
    Complex &  operator/=(Complex &);
    Complex &  operator/=(double);
    Complex  operator+();
    Complex  operator-();

    string format();
    string phasor();

// Implementation
private:
        double re, im;
};


// Inline Complex functions

inline Complex::Complex(double re_val, double im_val)
{
    re = re_val;
    im = im_val;
}

inline Complex::Complex()
{
/* if you want your Complex numbers initialized ...
    re = im = 0;
*/
}

inline Complex  Complex::operator+()
{
    return *this;
}

inline Complex Complex::operator-()
{
    return Complex(-re, -im);
}


// Definitions of compound-assignment operator member functions

inline Complex & Complex::operator+=(Complex & z2)
{
    re += z2.re;
    im += z2.im;
    return *this;
}

inline Complex & Complex::operator+=(double re_val2)
{
    re += re_val2;
    return *this;
}

inline Complex & Complex::operator-=(Complex & z2)
{
    re -= z2.re;
    im -= z2.im;
    return *this;
}

inline Complex & Complex::operator-=(double re_val2)
{
    re -= re_val2;
    return *this;
}

inline Complex &  Complex::operator*=(double re_val2)
{
    re *= re_val2;
    im *= re_val2;
    return *this;
}

inline Complex &  Complex::operator/=(double re_val2)
{
    re /= re_val2;
    im /= re_val2;
    return *this;
}


// Definitions of non-member Complex functions

inline double real(Complex & z)
{
    return z.re;
}

inline double imag(Complex & z)
{
    return z.im;
}

inline Complex conj(Complex & z)
{
    return Complex(z.re, -z.im);
}

inline double norm(Complex &x)     // z = x*conj(x) or  |x|^2
{
	return x.re*x.re+x.im*x.im;
}



inline Complex Complex::polar(double mag, double angle)
{
    angle /= dgrad;
    return Complex(mag*cos(angle), mag*sin(angle));
}


// Definitions of non-member binary operator functions

inline Complex operator+(Complex const & z1, Complex const & z2)
{
    return Complex(z1.re + z2.re, z1.im + z2.im);
}

inline Complex operator+(double  re_val1, Complex const & z2)
{
    return Complex(re_val1 + z2.re, z2.im);
}

inline Complex  operator+(Complex const & z1, double re_val2)
{
    return Complex(z1.re + re_val2, z1.im);
}

inline Complex  operator-(Complex const & z1, Complex const & z2)
{
    return Complex(z1.re - z2.re, z1.im - z2.im);
}

inline Complex  operator-(double re_val1, Complex const  & z2)
{
    return Complex(re_val1 - z2.re, -z2.im);
}

inline Complex  operator-(Complex const & z1, double re_val2)
{
    return Complex(z1.re - re_val2, z1.im);
}

inline Complex  operator*(Complex const & z1, double re_val2)
{
    return Complex(z1.re*re_val2, z1.im*re_val2);
}

inline Complex  operator*(double re_val1, Complex const & z2)
{
    return Complex(z2.re*re_val1, z2.im*re_val1);
}

inline Complex  operator/(Complex const & z1, double re_val2)
{
    return Complex(z1.re/re_val2, z1.im/re_val2);
}

inline int  operator==(Complex const & z1, Complex const & z2)
{
    return z1.re == z2.re && z1.im == z2.im;
}

inline int  operator!=(Complex const & z1, Complex const & z2)
{
    return z1.re != z2.re || z1.im != z2.im;
}

#if defined(STREAM_IO)
// Complex stream I/O

ostream &   operator<<(ostream &, Complex &);
istream &   operator>>(istream &, Complex &);
#endif


